insert into tcs_gravity_event_annotations (transient_object_id,gravity_event_id,gracedb_id,enclosing_contour,map_name,map_iteration,days_since_event,probability,dateLastModified,updated,dateCreated) values
(1122541681071342300,'S230518h','S230518h',60,'ligo-skymap-from-samples.multiorder.fits','20230518T165232_update',1.1839,51.8,'2023-05-24 12:27:12',0,'2023-05-24 12:27:12');

