alter table atlas_detectionsddc engine=InnoDB;
alter table atlas_diff_detections engine=InnoDB;
alter table atlas_diff_logs engine=InnoDB;
alter table atlas_diff_moments engine=InnoDB;
alter table atlas_diff_objects engine=InnoDB;
alter table atlas_diff_subcell_logs engine=InnoDB;
alter table atlas_diff_subcells engine=InnoDB;
alter table atlas_forced_photometry engine=InnoDB;
alter table atlas_heatmaps engine=InnoDB;
alter table atlas_metadata engine=InnoDB;
alter table atlas_metadataddc engine=InnoDB;
alter table atlas_stacked_forced_photometry engine=InnoDB;
alter table tcs_catalogue_tables engine=InnoDB;
alter table tcs_cfa_detections engine=InnoDB;
alter table tcs_cfa_to_ipp_lookup engine=InnoDB;
alter table tcs_classification_flags engine=InnoDB;
alter table tcs_classification_history engine=InnoDB;
alter table tcs_cmf_metadata engine=InnoDB;
alter table tcs_cross_matches engine=InnoDB;
alter table tcs_cross_matches_external engine=InnoDB;
alter table tcs_detection_lists engine=InnoDB;
alter table tcs_followup_photometry engine=InnoDB;
alter table tcs_followup_telescope_instruments engine=InnoDB;
alter table tcs_followup_telescopes engine=InnoDB;
alter table tcs_forced_photometry engine=InnoDB;
alter table tcs_gravity_event_annotations engine=InnoDB;
alter table tcs_gravity_events engine=InnoDB;
alter table tcs_image_groups engine=InnoDB;
alter table tcs_images engine=InnoDB;
alter table tcs_ipp_to_cfa_lookup engine=InnoDB;
alter table tcs_latest_object_stats engine=InnoDB;
alter table tcs_object_comments engine=InnoDB;
alter table tcs_object_group_definitions engine=InnoDB;
alter table tcs_object_groups engine=InnoDB;
alter table tcs_parameter_definitions engine=InnoDB;
alter table tcs_postage_stamp_images engine=InnoDB;
alter table tcs_postage_stamp_requests engine=InnoDB;
alter table tcs_postage_stamp_status_codes engine=InnoDB;
alter table tcs_processing_status engine=InnoDB;
alter table tcs_search_parameters engine=InnoDB;
alter table tcs_tns_requests engine=InnoDB;
alter table tcs_transient_objects engine=InnoDB;
alter table tcs_transient_reobservations engine=InnoDB;
alter table tcs_zoo_requests engine=InnoDB;
alter table tcs_zooniverse_scores engine=InnoDB;
